﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class EvaluationService : IEvaluationService
    {
        private readonly Tsi1Context _context;
        private readonly IMapper _mapper;
        private readonly IFileService _fileService;

        public EvaluationService(Tsi1Context context, IMapper mapper, IFileService fileService)
        {
            _context = context;
            _mapper = mapper;
            _fileService = fileService;
        }

        public async Task<ServiceResult<bool>> CancelRegistration(int evaluationId, int userId)
        {
            var result = new ServiceResult<bool>();

            var validation = await this.StudentValidation(evaluationId, userId);

            if (validation.HasError)
            {
                result.HasError = true;
                result.AddMessage(validation.Message);
                return result;
            }

            var studentId = validation.Data;

            var evaluationInscription = await _context.EvaluationInscriptions
                .FirstOrDefaultAsync(x => x.EvaluationId == evaluationId && x.StudentId == studentId);

            if (evaluationInscription == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.EvaluationInscriptionDoesNotExists, studentId, evaluationId));
                return result;
            }

            if (evaluationInscription.HasAttended)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.EvaluationInscriptionHasAttended, studentId, evaluationId));
                return result;
            }

            _context.EvaluationInscriptions.Remove(evaluationInscription);

            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<bool>> DeleteLaboratory(int submissionId, int userId)
        {
            var result = new ServiceResult<bool>();

            var user = await _context.Users
                .AsNoTracking()
                .Include(x => x.Student)
                .FirstOrDefaultAsync(x => x.Id == userId);

            if (user == null || user.Student == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.UserDoesNotExist, userId));
                return result;
            }

            var studentId = user.StudentId;

            var submission = await _context.Submissions.FirstOrDefaultAsync(x => x.Id == submissionId);

            if (submission == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.SubmissionDoesNotExists, submissionId));
                return result;
            }

            if (submission.StudentId != studentId)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.InavalidUserSubmission, submissionId, studentId));
                return result;
            }

            _context.Submissions.Remove(submission);

            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<List<EvaluationInscriptionDto>>> GetAllEvaluationInscriptions(int userId, int evaluationId)
        {
            var result = new ServiceResult<List<EvaluationInscriptionDto>>();

            var validation = await this.ProfessorValidation(evaluationId, userId);

            if (validation.HasError)
            {
                result.HasError = true;
                result.AddMessage(validation.Message);
                return result;
            }

            var evaluationInscriptions = await _context.EvaluationInscriptions
                .Where(x => x.EvaluationId == evaluationId)
                .ToListAsync();

            result.Data = _mapper.Map<List<EvaluationInscriptionDto>>(evaluationInscriptions);
            return result;
        }

        public async Task<ServiceResult<List<MyEvaluationInscriptionDto>>> GetMyEvaluationInscriptions(int userId, int courseId)
        {
            var result = new ServiceResult<List<MyEvaluationInscriptionDto>>();

            var course = await _context.Courses
                .AsNoTracking()
                .FirstOrDefaultAsync(x => x.Id == courseId);

            if (course == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.CourseDoesNotExist, courseId));
                return result;
            }

            var user = await _context.Users
                .AsNoTracking()
                .Include(x => x.Student)
                .FirstOrDefaultAsync(x => x.Id == userId);

            if (user == null || user.Student == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.UserDoesNotExist, userId));
                return result;
            }

            var studentCourse = await _context.StudentCourses
                .AsNoTracking()
                .FirstOrDefaultAsync(x => x.StudentId == user.StudentId && x.CourseId == courseId);

            if (studentCourse == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.StudentCourseDoesNotExists, user.StudentId, courseId));
                return result;
            }

            var evaluationInscriptions = await _context.EvaluationInscriptions
                .AsNoTracking()
                .Include(x => x.Evaluation)
                    .ThenInclude(x => x.Activity)
                .Include(x => x.Evaluation)
                    .ThenInclude(x => x.EvaluationType)
                .Where(x => x.StudentId == user.StudentId)
                .ToListAsync();

            var myEvaluationInscriptionDtos = new List<MyEvaluationInscriptionDto>();

            foreach (var evaluationInscription in evaluationInscriptions)
            {
                var myEvaluationInscriptionDto = new MyEvaluationInscriptionDto
                {
                    ActivityName = evaluationInscription.Evaluation.Activity.Name,
                    EvaluationTypeId = evaluationInscription.EvaluationId,
                    EvaluationType = evaluationInscription.Evaluation.EvaluationType.Name,
                    Grade = evaluationInscription.Grade,
                    HasAttended = evaluationInscription.HasAttended,
                };

                myEvaluationInscriptionDtos.Add(myEvaluationInscriptionDto);
            }

            result.Data = myEvaluationInscriptionDtos;
            return result;
        }

        public async Task<ServiceResult<bool>> QualifyEvaluationInscription(EvaluationInscriptionModifyDto evaluationInscriptionDto, int userId)
        {
            var result = new ServiceResult<bool>();

            var evaluationId = evaluationInscriptionDto.EvaluationId;
            var studentId = evaluationInscriptionDto.StudentId;

            var validation = await this.ProfessorValidation(evaluationId, userId);

            if (validation.HasError)
            {
                result.HasError = true;
                result.AddMessage(validation.Message);
                return result;
            }         

            var evaluationInscription = await _context.EvaluationInscriptions
                .FirstOrDefaultAsync(x => x.EvaluationId == evaluationId && x.StudentId == studentId);

            if (evaluationInscription == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.EvaluationInscriptionDoesNotExists, studentId, evaluationId));
                return result;
            }

            evaluationInscription.Grade = evaluationInscriptionDto.Grade;
            evaluationInscription.HasAttended = true;

            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<bool>> Registration(int evaluationId, int userId)
        {
            var result = new ServiceResult<bool>();

            var validation = await this.StudentValidation(evaluationId, userId);

            if (validation.HasError)
            {
                result.HasError = true;
                result.AddMessage(validation.Message);
                return result;
            }

            var studentId = validation.Data;

            var evaluationInscription = await _context.EvaluationInscriptions
                .AsNoTracking()
                .FirstOrDefaultAsync(x => x.EvaluationId == evaluationId && x.StudentId == studentId);

            if (evaluationInscription != null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.EvaluationInscriptionAlreadyExists, studentId, evaluationId));
                return result;
            }

            var newEvaluationInscription = new EvaluationInscription
            {
                EvaluationId = evaluationId,
                StudentId = studentId,
                Grade = 0,
                HasAttended = false,
            };

            _context.EvaluationInscriptions.Add(newEvaluationInscription);

            await _context.SaveChangesAsync();

            return result;
        }

        public async Task<ServiceResult<bool>> SubmitLaboratory(FileDto fileDto, int evaluationId, int userId)
        {
            var result = new ServiceResult<bool>();

            var validation = await this.StudentValidation(evaluationId, userId);

            if (validation.HasError)
            {
                result.HasError = true;
                result.AddMessage(validation.Message);
                return result;
            }

            var studentId = validation.Data;

            var evaluation = await _context.Evaluations
                .Include(x => x.Submissions)
                .FirstOrDefaultAsync(x => x.Id == evaluationId);

            var studentIds = evaluation.Submissions.Select(x => x.StudentId);

            if (studentIds.Contains(studentId))
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.SubmissionAlreadyExists, studentId, evaluationId));
                return result;
            }

            if (!_fileService.ExistFile(fileDto.Path))
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.FileDoesNotExist, fileDto.Path));
                return result;
            }

            var file = _mapper.Map<File>(fileDto);

            var newSubmission = new Submission
            {
                File = file,
                StudentId = studentId,
                Grade = 0,
                IsCompleted = false,
            };

            evaluation.Submissions.Add(newSubmission);

            await _context.SaveChangesAsync();

            return result;
        }

        private async Task<ServiceResult<int>> StudentValidation(int evaluationId, int userId)
        {
            var result = new ServiceResult<int>();

            var evaluation = await _context.Evaluations
                .AsNoTracking()
                .Include(x => x.Activity)
                .FirstOrDefaultAsync(x => x.Id == evaluationId);

            if (evaluation == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.EvaluationDoesNotExist, evaluationId));
                return result;
            }

            if (evaluation.IsCompleted)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.EvaluationIsCompleted, evaluationId));
                return result;
            }

            var user = await _context.Users
                .AsNoTracking()
                .Include(x => x.Student)
                .FirstOrDefaultAsync(x => x.Id == userId);

            if (user == null || user.Student == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.UserDoesNotExist, userId));
                return result;
            }

            var activityCourseId = evaluation.Activity.CourseId;

            var studentCourse = await _context.StudentCourses
                .AsNoTracking()
                .FirstOrDefaultAsync(x => x.CourseId == activityCourseId && x.StudentId == user.StudentId);

            if (studentCourse == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.StudentCourseDoesNotExists, user.StudentId, activityCourseId));
                return result;
            }

            result.Data = (int)user.StudentId;
            return result;
        }

        private async Task<ServiceResult<int>> ProfessorValidation(int evaluationId, int userId)
        {
            var result = new ServiceResult<int>();

            var evaluation = await _context.Evaluations
                .AsNoTracking()
                .Include(x => x.Activity)
                .FirstOrDefaultAsync(x => x.Id == evaluationId);

            if (evaluation == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.EvaluationDoesNotExist, evaluationId));
            }

            var user = await _context.Users
                .AsNoTracking()
                .Include(x => x.Professor)
                .FirstOrDefaultAsync(x => x.Id == userId);

            if (user == null || user.Professor == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.ProfessorDoesNotExist, userId));
                return result;
            }

            var activityCourseId = evaluation.Activity.CourseId;

            var professorCourse = await _context.ProfessorCourses
                .AsNoTracking()
                .FirstOrDefaultAsync(x => x.CourseId == activityCourseId && x.ProfessorId == user.ProfessorId);

            if (professorCourse == null)
            {
                result.HasError = true;
                result.AddMessage(string.Format(ErrorMessages.ProfessorCourseDoesNotExists, user.ProfessorId, activityCourseId));
                return result;
            }

            result.Data = (int)user.ProfessorId;
            return result;
        }
    }
}
